import fs from "fs/promises";
import path from "path";
import { createHeader } from "./header.js";
import Logger from "@home-gallery/logger";
const log = Logger("events.append");
const stringifyNdJson = (value) => JSON.stringify(value) + "\n";
const writeEvents = async (eventsFilename, events) => {
  const t0 = Date.now();
  const exists = await fs.access(eventsFilename).then(() => true).catch(() => false);
  if (!exists) {
    await fs.mkdir(path.dirname(eventsFilename.toString()), { recursive: true });
  }
  const data = stringifyNdJson(createHeader()) + events.map(stringifyNdJson).join("");
  await fs.writeFile(eventsFilename, data);
  log.debug(t0, `Wrote ${events.length} events to ${eventsFilename}`);
};
const appendEvents = async (eventsFilename, events) => {
  const t0 = Date.now();
  const stat = await fs.stat(eventsFilename).catch((err) => err.code == "ENOENT" ? false : Promise.reject(err));
  if (!stat || stat?.size === 0) {
    log.debug(`Events file ${eventsFilename} is missing or empty. Write new file`);
    return writeEvents(eventsFilename, events);
  }
  await fs.appendFile(eventsFilename, events.map(stringifyNdJson).join(""));
  log.debug(t0, `Append ${events.length} events to ${eventsFilename}`);
};
const appendEvent = (eventsFilename, event) => appendEvents(eventsFilename, [event]);
export {
  appendEvent,
  appendEvents,
  writeEvents
};
//# sourceMappingURL=append-event.js.map
