import Logger from "@home-gallery/logger";
import { SemVer } from "./semver.js";
const log = Logger("plugin.resolve");
function mapName(plugins) {
  return plugins.reduce((result, ctx) => {
    result[ctx.plugin.name] = ctx;
    return result;
  }, {});
}
function swap(names, from, to) {
  const tmp = names[to];
  names[to] = names[from];
  names[from] = tmp;
}
function findMissingDep(deps, names, name2plugin) {
  return deps.find((nameValue) => {
    const [name, requiredVersion] = nameValue.split("@");
    if (!names.includes(name)) {
      return nameValue;
    }
    if (requiredVersion) {
      const pluginVersion = name2plugin[name].plugin.version;
      const pluginSemVer = new SemVer(pluginVersion);
      return !pluginSemVer.matches(new SemVer(requiredVersion));
    }
    return false;
  });
}
function resolve(plugins) {
  const name2plugin = mapName(plugins);
  const names = plugins.map((ctx) => ctx.plugin.name);
  const orderedNames = [];
  let pos = 0;
  let swappedPos = 1;
  while (pos < names.length) {
    const name = names[pos];
    const deps = name2plugin[name].plugin.requires || [];
    const missingDep = findMissingDep(deps, orderedNames, name2plugin);
    if (!missingDep) {
      orderedNames.push(name);
      pos++;
      swappedPos = pos + 1;
      continue;
    }
    if (swappedPos == names.length) {
      const err = new Error(`Could not resolve dependency of ${name}. Missing required dependency ${missingDep}`);
      Object.assign(err, {
        name,
        missingDep
      });
      throw err;
    }
    swap(names, pos, swappedPos++);
  }
  const orderedPlugins = orderedNames.map((name) => name2plugin[name]);
  return orderedPlugins;
}
const resolveValid = (plugins) => {
  try {
    return resolve(plugins);
  } catch (err) {
    if (!err.name) {
      throw err;
    }
    log.warn(`Failed to resolve plugin dependency. Plugin ${err.name} requires missing dependency ${err.missingDep}. Skip plugin`);
    const excludeFailing = plugins.filter((plugin) => plugin.plugin.name != err.name);
    return resolveValid(excludeFailing);
  }
};
export {
  resolve,
  resolveValid
};
//# sourceMappingURL=resolve.js.map
